"""Specification for the plugin subsystem

We're using `pluggy <https://pluggy.readthedocs.io/en/latest/>`_ as the plugin
framework.
"""
import logging
import os
import sys
from typing import Iterable, Optional

import pkg_resources
import pluggy
from click import Command as ClickCommand


try:
    from dataclasses import dataclass
except ModuleNotFoundError:
    from attr import dataclass  # type: ignore


logger = logging.getLogger(__name__)


__all__ = ["Command", "add_commands", "plugin_manager", "hookimpl"]

PROJECT_NAME = "gamma.cli"

hookspec = pluggy.HookspecMarker(PROJECT_NAME)
hookimpl = pluggy.HookimplMarker(PROJECT_NAME)


@dataclass
class Command:
    """Class declaring a wrapped Command

    Attributes:
        command: A function wrapped with ``click.command`` or  ``click.group``
            decorators
        name: The command name. If not specified, it's taken from the function name.
        parent: The parent ``click.group``. If ``None``, install the command on the
            root level. You can use `` `` *(spaces)* to specify nested groups.
            Ie. ``foo bar`` to put the command under the ``foo``,  ``bar`` groups.
    """

    command: ClickCommand
    name: Optional[str] = None
    parent: Optional[str] = None


@hookspec
def add_commands() -> Iterable[Command]:
    """Add commands to the CLI.

    You must return a list of ``Command`` instances. Check the examples in
    the `gamma.cli.commands` package.

    You may use a ``.`` (dot) notation to append commands to existing groups. For
    example, (``new.foo`` will add the ``foo`` command to the ``new`` group)
    """


def load_entrypoints():
    """Use entry_points to load plugin modules.

    Plugins modules are loaded using standard ``pkg_resources.iter_entry_points``
    infrastructure.

    The best practice to setup application logging is using a plugin itself. However,
    you can enable logging during plugin initialization by setting up the
    `GAMMA_PLUGINS_LOGLEVEL` variable to a given loglevel. Due to that, logging
    plugins should check for a root handler with `_gamma == True` attribute set.
    """

    log_level = os.getenv("GAMMA_PLUGINS_LOGLEVEL")
    if log_level:
        logging.basicConfig(level=log_level.upper())

        # tag all root handlers
        for handler in logging.root.handlers:
            setattr(handler, "_gamma", True)

    for entrypoint in pkg_resources.iter_entry_points(PROJECT_NAME):
        logger.info("Loading plugin: %s", entrypoint.name)
        entrypoint.load()


# create a manager and add the spec
plugin_manager = pluggy.PluginManager(PROJECT_NAME)
plugin_manager.add_hookspecs(sys.modules[__name__])
