"test github module"
import os
import sys
import random
import string
from pathlib import Path
from github import BadCredentialsException
import pytest
import gamma.cli.services.github as github

PACKAGE_PATH = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, PACKAGE_PATH + "/..")


@pytest.fixture(name="token_path")
def handle_token_file():
    "provides the name of a temp file, with the garantee that it will be deleted at the end of the test"
    # azure fix on windows vm tests: no access to temp file
    tmp_dir = os.getcwd() + "/tmp/"
    if not os.path.exists(tmp_dir):
        os.makedirs(tmp_dir)
    try:
        token_f = (
            # tempfile.gettempdir()
            tmp_dir
            + "/fake_token-"
            + "".join(random.sample(string.ascii_letters, 8))
        )
        yield token_f
    finally:
        if os.path.exists(token_f):
            os.remove(token_f)


# The token we use for checking the account is a very special token, with very limited rights. it is associated with anselme vignon acount,
#    and will fail when this account is disabled.
#    to create a 'valid' replacement token,
#      - got to: https://github.gamma.bcg.com/settings/tokens
#      - create a token, with ONLY the scope "read-user" and "read:public_key" activated (anything else would be a security concern).
#      - copy paste the generated token into the "test_token" file
with open("tests/test_token", "r") as f:
    LIMITED_ACCESS_TOKEN = f.read()


@pytest.fixture(name="svc")
def valid_svc(token_path):
    "fixture providing a github service, with token file written with the LIMITED ACCESS TOKEN"
    with open(token_path, "w+") as fwrite:
        fwrite.write(LIMITED_ACCESS_TOKEN)
    return github.GitHubService(token_path=token_path)


def test_git_path():
    "Check that the service actually have a correct token path"
    home = str(Path.home())
    token_path = os.path.join(home, ".gamma", "github_token")

    assert github.get_token_path() == token_path


def test_read_token(token_path):
    "check that we read a token from file"
    token = "atoken"
    with open(token_path, "w+") as t_file:
        t_file.write(token)
    assert github.read_token(token_path) == token


def test_git_return_user_fromtoken():
    "check that when reading a token, gamma-cli logs to github, and performl a test call to github"


def test_createtoken_failgracefully_on_invalidtoken(token_path):
    "check that when no token, or an invalid token is provided, gamma-cli fails gracefully"
    fake_token = "not_a_valid_token"
    try:
        github.create_token(path=token_path, token=fake_token)
        assert 1 == 0, "I should have failed before"
    except BadCredentialsException:  #  pylint:disable=broad-except
        assert not os.path.exists(token_path), "token creation is guarded from error"


def test_createtoken_writetoken_on_success(token_path):
    """check that the token is written when initialized with the correct token.
    """
    github.create_token(path=token_path, token=LIMITED_ACCESS_TOKEN)
    assert os.path.exists(token_path)


def test_git_fail_on_wrong_user_token(token_path):
    "checks that gamma fails gracefully when writing a wrong token, and do not write the token"
    try:
        github.GitHubService(token="fake_token", token_path=token_path)
    except BadCredentialsException:  #  pylint:disable=broad-except
        assert not os.path.exists(token_path), "token creation is guarded from error"


def test_git_user(svc):
    "test that Githubservice can return a valid user"
    user = svc.user
    assert (
        user.login == "Vignon-Anselme"
    ), (
        "wrong user"
    )  # 10% sure this will not cause issues in the future. please change when changing the token
    assert user.login == user.login, "problem with the caching..."


def test_git_client(svc):
    "test that Githubservice can return a valid client"
    assert svc.client is not None
    assert svc.client == svc.client, "problem with the caching..."


def test_git_token(svc):
    "test that Githubservice can return a token"
    assert svc.token == LIMITED_ACCESS_TOKEN, "wrong token returned"
    assert svc.token == svc.token, "problem with caching the token"
