"""
Github services used to connect and handle all the operations with the Github API
"""

import os
from typing import Optional
from pathlib import Path
from getpass import getpass
from termcolor import cprint
from github import Github, NamedUser, GithubException
from github.GithubException import BadCredentialsException


def get_token_path() -> str:
    """ Returns the path of the token on the disk """

    home = str(Path.home())
    default_token_path = os.path.join(home, ".gamma", "github_token")
    return os.getenv("GITHUB_TOKEN_PATH", default_token_path)


def read_token(path: str) -> str:
    """ read the token from the file """
    with open(path, "r") as file:
        return file.read()


def delete_token_file():
    """ Deletes the token file """

    token_path = get_token_path()
    if os.path.exists(token_path):
        os.remove(token_path)


def create_token(path: str, token: str = None) -> str:
    """ prompt for the token then save it in a file """

    print(
        """In order to connect your project to your GitHub account, we need you to give us access
        to it through a personal access token. In order to do so:
        1. Visit https://github.gamma.bcg.com/settings/tokens
        2. Name the personal access token ("gamma-cli token" for example)
        3. Activate all the available scopes, except "admin:gpg_key"
        4. Generate the token
        """
    )
    if not token:
        token = (
            getpass("Copy the personal access token and paste it here: ")
            .lower()
            .strip()
        )
    validate_token(token)
    os.makedirs(os.path.dirname(path), exist_ok=True)
    with open(path, "w") as file:
        file.write(token)
    return token


def validate_token(token: str, svc: Github = None):
    "check that a token is valid, by calling its user endpoint. If a service is provided, it is checked instead of the token"
    if not svc:
        svc = GitHubService(token=token)
    try:
        user = svc.user
        cprint(
            "You have successfully logged in with the username: " + user.login, "green"
        )
    except BadCredentialsException:
        cprint("error validating the token: {}".format(token), "red")
        raise
    except GithubException:
        cprint("error validating the token: {}".format(token), "red")
        raise


class GitHubService(object):
    """ Github service """

    def __init__(self, token: str = None, token_path=None):
        "init with empty cached values. 'token' and 'token_path' parameters are used to preload values, for testing purposes"
        self._token = token
        self._user = None
        self._client = None
        self._token_path = token_path

    def warn_ssh_key(self) -> None:
        """ warn if no ssh key is associated with the token """
        try:
            keys = self.user.get_keys()
            number_of_key = 0
            for _ in keys:
                number_of_key += 1
            if number_of_key == 0:
                cprint(
                    "You should generate a ssh key and add it to your account. "
                    "Please visit (https://github.gamma.bcg.com/settings/keys) to generate a key",
                    "yellow",
                )
        except GithubException:
            cprint(
                "no SSH keys configured, not mandatory but nice to have: https://github.gamma.bcg.com/settings/keys",
                "yellow",
            )

    def set_token(self) -> str:
        """ get the token from the file or prompt for it"""
        token_path = get_token_path() if not self._token_path else self._token_path
        token_file = Path(token_path)
        token = ""
        if token_file.is_file():
            self._token = read_token(token_path)
        else:
            self._token = create_token(token_path)
        self.warn_ssh_key()
        return token

    def get_github_client(self) -> Github:
        """ Create a github client from the token """
        if not self._client:
            self._client = Github(
                base_url="https://github.gamma.bcg.com/api/v3", login_or_token=self.token
            )
        return self._client

    def cached_token(self) -> Optional[str]:
        "return a cached token"
        if not self._token:
            self.set_token()
        return self._token

    def get_user(self) -> NamedUser:
        "return a cached user, fetching it if needed"
        if not self._user:
            self._user = self.client.get_user()
        return self._user

    # lighter syntax
    token = property(fget=cached_token)
    client = property(fget=get_github_client)
    user = property(fget=get_user)
