"""Module for registering pip repository providers
"""
import sys
from typing import Any, Dict, NamedTuple

import pluggy


PROJECT_NAME = "gamma.cli.services.pip"

hookspec = pluggy.HookspecMarker(PROJECT_NAME)
hookimpl = pluggy.HookimplMarker(PROJECT_NAME)


class PyPIRepository(NamedTuple):

    # The provider key
    provider: str

    # The repository name
    name: str

    # A description of the repository
    description: str

    # A templatable URI for the repository. The URI specification is provider specific
    repo: str

    # If True, will automatically setup the repository
    auto_setup: bool


@hookspec
def install_repo(
    repo: PyPIRepository, pip_setup, session: Dict[str, Any]
):
    """Install a repository into pip.conf.

    This function may be called multiple times during a installation session,
    once per repository. It's OK (and expected) to interactively guide the user
    via the CLI.

    Implementations should check the ``repo.provider`` attribute to find out if they're
    the intended audience for the installation and return ``True``, otherwise they
    should no-op and return ``False``.

    It's expected that this function call should add or updated repositories to match
    the specifications.

    Args:
        repo: Repository details.
        pip_setup: An instance of PipSetupService
        session: Dict that is kept across hook calls, where implementations can store
            data, ie. caching tokens.
    Return:
        ``True`` if the ``repo.provider`` matches this implementation, ``False`` otherwise.
    """
    raise NotImplementedError


# create a manager and add the specs
plugin_manager = pluggy.PluginManager(PROJECT_NAME)
plugin_manager.add_hookspecs(sys.modules[__name__])
