"""
Plugin allowing the generation of .gitignore files.

Using http://gitignore.io API.

example:

`gamma gitignore options`

will return you the list of available keywords:

- java
- python
- flask
- ...

`gamma gitignore generate python flask > .gitignore`

will generate a .gitignore file with all the common settings for python
and flask

"""

import logging
import re
import sys

import click
import requests
from termcolor import cprint



logger = logging.getLogger(__name__)



class GitIgnore(object):
    """
    GitIgnore module
    """

    telemetry = None

    def __init__(self, telemetry):
        self.telemetry = telemetry

    def get_options_raw(self):
        """ Downloads the list of options available from gitignore.io """

        url = "https://www.gitignore.io/api/list"
        try:
            response = requests.get(url, timeout=5)
            response.raise_for_status()
        except (requests.Timeout, requests.HTTPError) as e:
            logger.debug(e)
            cprint(
                "Could not connect to gitignore website. Please try again later", "red"
            )
            sys.exit(69)  # according to 'man sysexits', 69 is EX_UNAVAILABLE

        return response.content.decode("utf-8")

    def get_options(self):
        """ Get the options and parse them """

        raw = self.get_options_raw()
        data = raw.replace("\n", ",").split(",")
        data.sort()
        return data

    def options(self) -> int:
        """ get the options and display them """

        data = self.get_options()
        for opt in data:
            print(opt)

        return len(data)

    def get_file(self, option_list: str) -> str:
        """ generate the gitignore file from the API and display it """

        url = "https://www.gitignore.io/api/" + option_list

        try:
            response = requests.get(url, timeout=5)
        except requests.Timeout as e:
            logger.debug(e)
            cprint(
                "Could not connect to gitignore website. Please try again later", "red"
            )
            sys.exit(69)
        if response.status_code == 200:
            return response.content.decode("utf-8")

        body = response.content.decode("utf-8")
        errors = re.findall("!! ERROR: (\\w*) is undefined", body, re.IGNORECASE)
        print(
            "invalid keyword: "
            + " ".join(errors)
            + "\nYou can use:\n\tgamma gitignore"
            + "list\nto obtain the available keywords"
        )

        return ""

    def _split_options(self, types):
        return ",".join(types)

    def generate(self, types) -> bool:
        """ Prepare the options and request a call to the API """
        option_list = self._split_options(types)
        content = self.get_file(option_list)
        if content != "":
            print(content)
            if self.telemetry is not None:
                self.telemetry.track_event(
                    "gitignore generate options",
                    {"options": option_list, "success": True},
                )
            return True

        if self.telemetry is not None:
            self.telemetry.track_event(
                "gitignore generate options", {"options": option_list, "success": False}
            )

        return False


@click.group()
@click.pass_obj
def cli(telemetry):
    """
    Creates a starting .gitignore file.
    A .gitignore file will exclude certain file types from ever being added
    to the GitHub repository.

    For example, .csv or .xlsx data files are usually NOT committed to Git

    Request the "keyword" files related to your projects with the syntax:

    gamma gitignore generate python flask > .gitignore

    To get a list of available keywords:

    gamma gitignore options
    """

    if telemetry is not None:
        telemetry.track_event("gitignore cmd")
        telemetry.flush()


@cli.command()
@click.pass_obj
def options(telemetry):
    """
    Returns the list of available options
    """

    if telemetry is not None:
        telemetry.track_event("gitignore options cmd")

    module = GitIgnore(telemetry)
    module.options()

    if telemetry is not None:
        telemetry.flush()


@cli.command()
@click.pass_obj
@click.argument("types", nargs=-1, required=True)
def generate(telemetry, types):
    """
    Generates a .gitignore based on the keyword(s) provided.

    gamma gitignore generate python flask > .gitignore
    """

    if telemetry is not None:
        telemetry.track_event("gitignore generate cmd")

    module = GitIgnore(telemetry)
    module.generate(types)

    if telemetry is not None:
        telemetry.flush()
