"""
Plugin used to create new resources:

- Project using template and directly pushed to GitHub
- Create a Trello board
- Request a Virtual Machine
- ...

For example:

`gamma new project --path ~/home/projects/`

will create a new project in the `~/home/projects/` directory.

If the directory does not exist, a new directory will be created.

`gamma new trello`

will open the Trello login page to be used with your enterprise account.

`gamma new vm`

will open a request to the Gamma Platform team to obtain a VM

"""

import webbrowser

import click

from gamma.cli.services.project import ProjectService


@click.group()
@click.pass_obj
def cli(telemetry):
    """
    Allow you to create new resources
    """

    if telemetry is not None:
        telemetry.track_event("git new cmd")
        telemetry.flush()


@cli.command()
@click.option("--path", default=None, help="Directory location to create the project")
def project(path: str):
    """
    Will create a new project from the template and propose you to push it
    to Gamma GitHub directly
    """

    try:
        new_project = ProjectService()
        new_project.new(path)
    except Exception as ex:
        print(ex)
        exit(-1)


@cli.command()
def trello():
    """
    Open the web page to connect to Trello with your enterprise account,
    allowing you to create a new team/board
    """

    webbrowser.open("https://trello.com/login/sso", new=2, autoraise=True)


# pylint: disable=invalid-name
@cli.command()
def vm():
    """
    Open the web page to request a new VM from Gamma Platform
    """

    webbrowser.open(
        "https://jira.gamma.bcg.com/servicedesk/customer/portal/1/create/21",
        new=2,
        autoraise=True,
    )
