"""
Gamma toolbox helping to bootstrap new projects and all the common operations
needed along the lifecycle of projects
"""

import os
from collections import defaultdict
from functools import reduce
from operator import getitem, itemgetter

import click
import colorama
import pyfiglet
from termcolor import cprint

from gamma.cli import plugins
from gamma.cli.metadata import __version__
# pylint: disable=wrong-import-position
from gamma.cli.monitoring import enable_monitoring
from gamma.cli.updater import check_version
from gamma.cli.utils.fix_locale import fix_locale


colorama.init()

ASCII_BANNER = pyfiglet.figlet_format("Gamma")
cprint(ASCII_BANNER, "green")
cprint("gamma-cli ({}) is currently in beta".format(__version__), "yellow")
cprint(
    "please report any bug on: https://github.gamma.bcg.com/BCG/gamma-cli/issues",
    "yellow",
)
plugins.load_entrypoints()
print("")


class CLI(click.Group):
    """
    Define the CLI commands by using plugins.

    Plugins are specified in the ``plugins.py`` module.
    """

    def __init__(self, name, **attrs):

        super().__init__(name, {}, **attrs)

        import gamma.cli.commands.plugins  # NOQA

        cmds = []

        plugin_commands = plugins.plugin_manager.hook.add_commands()  # NOQA
        for plugin_block in plugin_commands:
            for cmd in plugin_block:
                if cmd.parent is None:
                    level = 0
                else:
                    level = len(cmd.parent.split()) + 1

                if cmd.name is not None:
                    cmd.command.name = cmd.name

                cmds.append((level, cmd))

        cmds.sort(key=lambda x: x[0])

        # Add groups to self
        for level, cmd in cmds:
            if level == 0 and isinstance(cmd.command, click.MultiCommand):
                self.add_command(cmd.command)

        # Add nodes
        for level, cmd in cmds:
            if level == 0:
                parent = self
            else:
                parents = cmd.parent.split(".")
                parent = reduce(self.get_command, parents, self)
            parent.add_command(cmd.command)


@click.version_option(version="0.2.1")
@click.command(cls=CLI)
@click.pass_context
@click.option("--no-telemetry", envvar="NO_TELEMETRY", is_flag=True)
def cli(ctx, no_telemetry):
    """
    Gamma Toolbox
    """

    fix_locale()
    ctx.obj = enable_monitoring(no_telemetry)
    check_version()


if __name__ == "__main__":
    # pylint: disable=no-value-for-parameter
    cli()
