import sys
from urllib.parse import quote_plus as urlquote, urlsplit

import click

from gamma.cli.services.pip.plugins import PyPIRepository, hookimpl, plugin_manager
from gamma.cli.services.pip.service import PipSetupService


@hookimpl
def install_repo(repo: PyPIRepository, pip_setup: PipSetupService, session):
    """Hook implementation for the ``artifactory-apikey`` provider"""

    if repo.provider != "artifactory-apikey":
        return False

    key = "artifactory-apikey::instance"
    provider = session.get(key)
    if not provider:
        provider = ArtifactoryApikeyProvider()
        session[key] = provider

    provider.install(pip_setup, repo)
    return True


class ArtifactoryApikeyProvider:
    """Repository setup provider for Artifactory, using the API Key auth method"""

    def __init__(self):
        self.credentials = {}

    def get_server(self, repo: PyPIRepository) -> str:
        """Get server:port info from a repo"""

        u = urlsplit(repo.repo.format(username="dummy", apikey="dummy"))
        host = u.hostname
        port = f":{u.port}" if u.port else ""
        return f"{host}{port}"

    def install(self, pip_setup: PipSetupService, repo: PyPIRepository) -> None:
        """Install a repository into user's pip conf"""

        server = self.get_server(repo)

        s_reponame = click.style(repo.name, fg="yellow")
        click.secho("\nSetting up artifactory PyPI repository: " + s_reponame)

        if server not in self.credentials:
            self.credentials[server] = self.fetch_credentials(server)

        creds = self.credentials[server]
        repo_url = repo.repo.format(**creds)
        pip_setup.ensure_extra_index_url(repo_url)
        click.secho("Done adding repo: " + s_reponame)

    def fetch_credentials(self, server):
        s_server = click.style(server, fg="yellow")
        click.secho(f"We need to fetch credentials for server [{s_server}]")
        click.secho(
            f"Please provide your username (lower case email for Artifactory): ",
            nl=False,
        )
        username = input()

        if not username:  # pragma: no cover
            raise click.ClickException("aborting, username not provided")

        profile_url = f"https://{server}/artifactory/webapp/#/profile"

        click.secho(
            f"\nArtifactory uses an API key to authenticate to the repository.\n"
            f'Please visit the URL below, copy and paste the value of the "API Key".\n'
            f"You may need to login first. If needed, click on your username on\n"
            f"the interface and create/copy the API Key.\n"
            f"\n    {profile_url}\n\n"
            f"API Key: ",
            nl=False,
        )
        apikey = input()
        if not apikey:  # pragma: no cover
            raise click.ClickException("aborting, apikey not provided")

        creds = {
            "username": urlquote(username.strip()),
            "apikey": urlquote(apikey.strip()),
        }

        return creds


plugin_manager.register(sys.modules[__name__])
