"""
Service to manipulate project, from the creation to the tuning
"""

import os
import webbrowser
from typing import List

from cookiecutter.exceptions import OutputDirExistsException
from cookiecutter.main import cookiecutter
from git import Repo
# pylint: disable=unused-import
from github import Github
from github.AuthenticatedUser import AuthenticatedUser
# pylint: enable=unused-import
from github.GithubException import GithubException
from github.Organization import Organization
from github.Repository import Repository
from termcolor import cprint
from tqdm import tqdm
from pathlib import Path
import shutil

from gamma.cli.services.github import GitHubService, delete_token_file
from gamma.cli.utils.input_number import input_int
from gamma.cli.utils.yes_or_no import yes_or_no


class ProjectService(object):
    """ Project service """

    github = GitHubService()
    github_client = None  # type: Github
    github_user = None  # type: AuthenticatedUser

    def __init__(self, gitHubService=None):
        if gitHubService is not None:
            self.github = gitHubService
        self.base_repo = "freier-niels/cookie-python.git"

    def _exec_cookiecutter(self, path: str):
        cprint(
            "Choose between beginner and expert mode. "
            "Beginner mode includes a config file, testing framework, environment management, and a gitignore file. "
            "Expert mode takes beginner mode and adds functionality for Docker, Tox, code coverage, mypy, and package management.",
            "green",
        )
        try:
            return cookiecutter(
                "https://" + self.github.token + "@github.gamma.bcg.com/" + self.base_repo,
                output_dir=path,
            )
        except OutputDirExistsException:
            cprint(
                "A project with the same name already exists in the directory", "red"
            )

        return None

    def _fix_new_path(self, path: str) -> str:
        cprint(
            "A new project will be created in {} ".format(
                "{}".format(os.path.abspath(path))
                if path is not None
                else "{}".format(os.getcwd())
            ),
            "green",
        )
        cprint("We will ask you few questions to customize it.", "green")

        if path is None:
            return "."

        return path

    def _get_orgs_where_user_admin(self) -> List[Organization]:
        orgs = self.github_user.get_orgs()
        number_of_org = 0
        orgs_user_is_admin = []
        number_of_org += orgs.totalCount
        cprint(
            "You are member of {} organisation(s), "
            "We will analyse where you are allowed to "
            "create a project".format(number_of_org),
            "green",
        )

        for org in tqdm(orgs, total=number_of_org, unit="org"):
            admins = org.get_members(role="admin")
            for admin in admins:
                if admin.login == self.github_user.login:
                    orgs_user_is_admin.append(org)

        return orgs_user_is_admin

    def _get_orgs_selector_option(self, orgs: List[Organization]) -> List[str]:
        org_options = []

        for org in orgs:
            if org.description is not None and org.description != "":
                org_options.append("{} - {}".format(org.login, org.description))
            else:
                org_options.append(org.login)
        org_options.append("Create a new organization for this project/case")
        org_options.append("{} - Your personal account".format(self.github_user.login))

        return org_options

    def _create_personal_repo(self, project_name: str) -> Repository:
        cprint(
            "The project will be created as {}/{}".format(
                self.github_user.login, project_name
            ),
            "green",
        )
        is_private = True
        if yes_or_no("Do you want set the project as public?"):
            is_private = False
        try:
            return self.github_user.create_repo(project_name, private=is_private)
        except GithubException:
            cprint(
                "{}/{} already exists".format(self.github_user.login, project_name),
                "red",
            )
            return None

    def _create_organisation_repo(self, org_name: str, project_name: str) -> Repository:
        cprint(
            "The project will be created in {}/{}".format(org_name, project_name),
            "green",
        )
        try:
            return self.github_client.get_organization(org_name).create_repo(
                project_name
            )
        except GithubException:
            cprint("{}/{} already exists".format(org_name, project_name), "red")
            return None

    def _push_base_project(self, repo: Repository, origin):
        repo.git.add(".")
        repo.git.commit(message="Base version from Gamma template")
        origin.push(refspec="master:master")

    def _decide_org_for_creation(self, project_name: str, options: List[str]) -> int:
        cprint(
            "Please choose where you want to create the project {}".format(
                project_name
            ),
            "green",
        )
        for i, opt in enumerate(options):
            print("{} - {}".format(i, opt))
        return input_int("Select the number of the organization you want to use: ")

    def _clean_cookiecutter_cache(self):
        """clean previous local copy of the repo, forcing to refresh it at each usage
        and not prompting to user
        """
        try:
            tmp_path = Path.home() / ".cookiecutters" / "cookie-python"
            if tmp_path.exists():
                shutil.rmtree(tmp_path)
        except:
            # not end of the world if not possible to delete,
            # cookiecutter will prompt to user as before
            pass

    def new(self, path: str):
        """ Will create a new project in the selected organization """
        path = self._fix_new_path(path)

        self._clean_cookiecutter_cache()

        repo_path = self._exec_cookiecutter(path)
        if repo_path is None:
            return
        project_name = os.path.basename(repo_path)

        repo = Repo.init(repo_path)
        if yes_or_no(
                "Do you want to create a repo on github.gamma.bcg.com for your project?"
        ):

            self.github_client = self.github.client
            self.github_user = self.github.user

            orgs_admin = []
            try:
                orgs_admin = self._get_orgs_where_user_admin()
            except Exception:
                delete_token_file()
                cprint(
                    "your github token doesn't allow to list the organization, please create a new token and use: gamma login",
                    "red",
                )
                raise
            options = self._get_orgs_selector_option(orgs_admin)
            index = self._decide_org_for_creation(project_name, options)

            if index == len(options) - 2:
                # create a new org then ask again to pick one
                webbrowser.open(
                    "https://github.gamma.bcg.com/organizations/new", new=2, autoraise=True
                )
                input("Continue?")
                orgs_admin = self._get_orgs_where_user_admin()
                options = self._get_orgs_selector_option(orgs_admin)
                index = self._decide_org_for_creation(project_name, options)

            if index == len(options) - 1:
                remote_repo = self._create_personal_repo(project_name)
            else:
                remote_repo = self._create_organisation_repo(
                    orgs_admin[index].login, project_name
                )
            if remote_repo is None:
                return

            origin = repo.create_remote(
                "origin",
                url=remote_repo.clone_url.replace(
                    "https://", "https://" + self.github.token + "@"
                ),
            )

            self._push_base_project(repo, origin)

            cprint("Your project can be found locally at " + repo_path, "green")
            cprint(
                "Your project can be found remotely at " + remote_repo.html_url, "green"
            )

        else:
            cprint(
                "Your project has been initialized with a template folder at "
                + repo_path,
                "green",
            )
            cprint(
                "Further support for BitBucket, GitLab, or other hosted Git services "
                "will be added later. Please use these services' `import repository` "
                "features to upload your code to the remote server.",
                "yellow",
            )
