"""
Module used to compare the release number and propose to the user to make an
update if needed with `pip3 install gamma-cli --upgrade`
"""

import requests
from termcolor import cprint
from version_parser import Version
import logging

from gamma.cli.metadata import __version__

logger = logging.getLogger(__name__)


GIST_RELEASE_URL = "https://gist.githubusercontent.com/stumpyfr/b4f89c297a4e479049ff463fed790b37/raw/version"


def _download_release_number(url: str) -> str:
    """
    try to download the latest release number from a secret gist and return a
    default version in case of any problem.
    """

    try:
        response = requests.get(url, timeout=5)
        response.raise_for_status()
        if response.status_code == 200:
            return response.content.decode("utf-8")
    except (
        requests.exceptions.MissingSchema,
        requests.exceptions.Timeout,
        requests.HTTPError,
    ) as e:
        logger.debug(e)
        cprint(
            "Unable to fetch latest gamma-cli version, please try again later", "red"
        )

    # if not possible to get the version, we default to a low version
    return "0.0.0"


def _compare_version(curr_version: Version, latest_version: Version) -> bool:
    if curr_version < latest_version:
        cprint(
            "a new version ({}) is available (current: {}), ".format(
                latest_version, curr_version
            )
            + "please update:\n\tpip3 install gamma-cli --upgrade",
            "yellow",
        )
        return False

    return True


def check_version() -> bool:
    """
    compare the latest release number and the current and indicate to the user
    it's time to update
    """

    latest_version = _download_release_number(GIST_RELEASE_URL)
    latest_version = Version(latest_version)
    curr_version = Version(__version__)

    return _compare_version(curr_version, latest_version)
