"""Module implemeting the `setup pip` service """
import sys
from pathlib import Path
from typing import Dict, List, Optional
from urllib.parse import urlsplit

import yaml

import gamma.cli

from . import plugins


class PipSetupService:
    """
    Pip configuration setup service

    Args:
        pip_config_file: If set, modify this file instead of the non-legacy user file

    """

    def __init__(self, pip_config_file=None):

        # load implementations
        from . import artifactory  # builtin implementaiton

        plugins.plugin_manager.load_setuptools_entrypoints(plugins.PROJECT_NAME)

        super().__init__()
        self._pip_config_file = pip_config_file
        self._repositories: Optional[List[plugins.PyPIRepository]] = None


    def get_repositories_yaml(self):
        """Look ``repositories.yaml`` file.

        It should be in either the root of the source tree (for development) or in
        ``{sys.prefix}/etc/gamma-cli/repositories.yaml``
        """

        repos_file = Path(gamma.cli.__file__).parents[2] / "repositories.yaml"
        if repos_file.exists():
            return repos_file

        repos_file = Path(sys.prefix) / "etc/gamma-cli/repositories.yaml"
        if repos_file.exists():
            return repos_file

        raise Exception("The repositories.yaml file was not found")


    @property
    def repositories(self) -> List[plugins.PyPIRepository]:
        if self._repositories is None:
            repos = self.get_repositories_yaml()
            with repos.open("r") as fo:
                _repos = yaml.load(fo, Loader=yaml.SafeLoader)["pip"]

            self._repositories = []
            for repo_dict in _repos:
                repo_dict.setdefault("auto_setup", False)
                repo = plugins.PyPIRepository(**repo_dict)
                self._repositories.append(repo)

        return self._repositories

    @property
    def pip_conf_file(self):
        if self._pip_config_file is None:
            from pip._internal.configuration import get_configuration_files

            # use the user scopd the "new" config file location
            self._pip_config_file = get_configuration_files()["user"][1]
        return self._pip_config_file

    def ensure_extra_index_url(self, new_extra_url):
        """Ensure an entry exists in ``pip.conf`` entry ``globals.extra-index-url`` """

        from configparser import RawConfigParser

        parser = RawConfigParser()
        if Path(self.pip_conf_file).exists():
            parser.read(self.pip_conf_file)
        else:
            parser.add_section("global")

        # get currently set extra-index-url values
        value = parser.get("global", "extra-index-url", fallback="")
        urls = [x.strip() for x in value.split("\n") if x.strip()]

        def get_key(url):
            # get test key to match for update
            _u = urlsplit(url)
            return (_u.scheme, _u.hostname, _u.port, _u.path, _u.query)

        new_key = get_key(new_extra_url)
        update_idx = None
        for i, url in enumerate(urls):
            key = get_key(url)
            if key == new_key:
                update_idx = i

        if update_idx is not None:
            urls[update_idx] = new_extra_url
        else:
            urls.append(new_extra_url)

        # write back the configuration
        urls = "\n".join(urls)
        parser.set("global", "extra-index-url", urls)
        with open(self.pip_conf_file, "w") as fo:
            parser.write(fo)

    def auto_install_repos(self):
        session = {}
        for repo in self.repositories:
            if repo.auto_setup:
                self.install_repo(repo, session)

    def install_repo(self, repo: plugins.PyPIRepository, session: Dict = None):
        if session is None:
            session = {}

        handled = plugins.plugin_manager.hook.install_repo(
            repo=repo, pip_setup=self, session=session
        )
        if not any(handled): # pragma: no cover
            raise ValueError(
                f"Error: no provider handling key '{repo.provider}' was "
                "found for repository named '{repo.name}'"
            )

    def get_repo(self, name):
        match = [x for x in self.repositories if x.name == name]
        if match:
            return match[0]
