import tempfile
from pathlib import Path
from unittest.mock import Mock

import pytest


@pytest.fixture
def pip_conf(monkeypatch):
    with tempfile.TemporaryDirectory() as td:
        pip_conf_file = f"{td}/pip.conf"
        yield pip_conf_file


def test_load_repositories():
    from gamma.cli.services.pip import service as pip_service

    obj = pip_service.PipSetupService()

    assert len(obj.repositories) > 1

    # look for the default gamma-pip repo
    gamma_pip = None
    for repo in obj.repositories:
        if repo.name == "gamma-pip":
            gamma_pip = repo

    assert gamma_pip is not None
    assert gamma_pip.provider == "artifactory-apikey"


def test_auto_install(pip_conf, monkeypatch):
    from gamma.cli.services.pip import service as pip_service
    from gamma.cli.services.pip import artifactory

    # mock user input to reply username and apikey
    mock_input = Mock(side_effect=["mockuser", "mockkey"])
    monkeypatch.setitem(artifactory.__dict__, "input", mock_input)

    obj = pip_service.PipSetupService(pip_config_file=pip_conf)
    obj.auto_install_repos()

    path = Path(pip_conf)
    assert path.exists()

    content = path.read_text()
    assert content

    # we know gamma-pip is installed automatically via the artifactory-apikey
    # plugin
    assert "gamma-pip" in content
    assert "mockuser:mockkey" in content


def test_install_all(pip_conf, monkeypatch):
    from gamma.cli.services.pip import service as pip_service
    from gamma.cli.services.pip import artifactory
    from configparser import RawConfigParser

    # mock user input to reply username and apikey
    mock_input = Mock(side_effect=["mockuser", "mockkey"])
    monkeypatch.setitem(artifactory.__dict__, "input", mock_input)

    obj = pip_service.PipSetupService(pip_config_file=pip_conf)
    session = {}
    for repo in obj.repositories:
        obj.install_repo(repo, session=session)

    path = Path(pip_conf)
    assert path.exists()

    parser = RawConfigParser()
    parser.read(pip_conf)
    urls = [
        x.strip()
        for x in parser.get("global", "extra-index-url").split("\n")
        if x.strip()
    ]

    assert len(urls) > 1

    match = False
    for url in urls:
        if "gamma-pip" in url:
            assert "mockuser:mockkey" in url
            match = True

    assert match

    # update a single entry in gamma-pip
    mock_input = Mock(side_effect=["newuser", "newkey"])
    monkeypatch.setitem(artifactory.__dict__, "input", mock_input)
    repo = obj.get_repo("gamma-pip")
    obj.install_repo(repo)

    parser = RawConfigParser()
    parser.read(pip_conf)
    new_urls = [
        x.strip()
        for x in parser.get("global", "extra-index-url").split("\n")
        if x.strip()
    ]

    # assert no new entry added
    assert len(new_urls) == len(urls)

    # check gamma-pip was updated
    match = False
    for url in new_urls:
        if "gamma-pip" in url:
            assert "newuser:newkey" in url
            match = True

    assert match


def test_user_pip_conf():
    from gamma.cli.services.pip import service as pip_service
    from pathlib import Path

    obj = pip_service.PipSetupService()

    # ensure relative to Home folder
    assert Path(obj.pip_conf_file).relative_to(Path.home())
